/*** Copyright: OGIHARA Takeshi, April 2008 ***/

#import "UroCtrl.h"
#import <Cocoa/Cocoa.h>
#import "UroPiece.h"
#import "UroView.h"

#define  NumberOfClients    15

@implementation UroCtrl

/* Local Method */
- (void)clearBoard
{
    int x, y;
    for (y = 0; y < BHeight; y++)
        for (x = 0; x < BWidth; x++)
            uBoard[y][x] = nil;
}

- (id)init
{
    if ((self = [super init]) != nil) {
        int i;
        proxies = [[NSMutableArray alloc] initWithCapacity:1];
        uBoard = (id **)malloc(sizeof(id *) * BHeight);
        for (i = 0; i < BHeight; i++)
            uBoard[i] = (id *)malloc(sizeof(id) * BWidth);
        [self clearBoard];
        uView = [[UroView alloc]
            initWithGrid:GridSize width:BWidth height:BHeight];
        [uView setBoard: uBoard];
        inAction = NO;
    }
    return self;
}

- (void)dealloc
{
    int i;
    for (i = 0; i < BHeight; i++)
        free((void *)uBoard[i]);
    free((void *)uBoard);
    [uView release];
    [proxies release];
    [super dealloc];
}

/* Local Method */
- (void)setupMainMenu
{
    NSMenuItem *item;
    NSMenu *menu, *submenu;

    menu = [[[NSMenu alloc] initWithTitle: AppName] autorelease];
    item = [menu addItemWithTitle:@"Action"
        action:@selector(submenuAction:) keyEquivalent:@""];
    submenu = [[[NSMenu alloc] initWithTitle:@"Action"] autorelease];
    [menu setSubmenu: submenu forItem:item];
    item = [submenu addItemWithTitle:@"New Board"
        action:@selector(resetBoard:) keyEquivalent:@"n"];
    [item setTarget:self];
    item = [submenu addItemWithTitle:@"Quit Urouro"
        action:@selector(terminate:) keyEquivalent:@"q"];
    // if target is nil, the message is sent to the first responder.
    [NSApp setMainMenu:menu];
}

/* Local Method */
- (void)windowSetUp
{
    NSRect rect = [uView frame];
    rect.origin = NSMakePoint(120.0, 150.0);
    uWindow = [[NSWindow alloc] initWithContentRect:rect
        styleMask: NSTitledWindowMask
        backing:NSBackingStoreBuffered defer:YES];
    [uWindow setReleasedWhenClosed:YES];
    [uWindow setTitle: AppName];
    [uWindow setContentView:uView];
    [uView prepareDrawing];
}

/* Local Method */
- (void)clientSetUp
{
    NSPort *port1;
    NSPort *port2;
    NSArray *portArray;
    NSConnection *conn;

    port1 = [NSPort port];
    port2 = [NSPort port];
    conn = [[NSConnection alloc]
        initWithReceivePort:port1 sendPort:port2];
    if (conn == nil)
        return;
    [conn setRootObject:self];
    portArray = [NSArray arrayWithObjects:port2, port1, nil];

    [NSThread detachNewThreadSelector:@selector(connectNewClient:)
        toTarget:[UroPiece class] withObject:portArray];
}

- (void)registerAsClient:(id)client {
    [client setProtocolForProxy:@protocol(UroClient)];
    [proxies addObject:client];
}

- (BOOL)moveFrom:(UroPoint)loc to:(UroPoint)newloc
{
    UroPoint lowleft, upright;

    if (!inAction || uBoard[newloc.y][newloc.x])
        return NO;
    uBoard[newloc.y][newloc.x] = uBoard[loc.y][loc.x];
    uBoard[loc.y][loc.x] = nil;
    if (loc.x > newloc.x)
        lowleft.x = newloc.x, upright.x = loc.x;
    else
        lowleft.x = loc.x, upright.x = newloc.x;
    if (loc.y > newloc.y)
        lowleft.y = newloc.y, upright.y = loc.y;
    else
        lowleft.y = loc.y, upright.y = newloc.y;

    [uView drawFrom:lowleft to:upright];
    return YES;
}

- (void)resetBoard:(id)sender
{
    UroPiece *obj;
    UroPoint loc;

    inAction = NO;
    for (obj in proxies)
        [obj stop];
    [self clearBoard];
    for (obj in proxies) {
        do {
            loc = [obj reset];
        }while (uBoard[loc.y][loc.x]);
        uBoard[loc.y][loc.x] = obj;
    }
    [uView prepareDrawing];
    [uView display];
    inAction = YES;
    for (obj in proxies)
        [obj start];
}

/* Application Delegate Message */
- (void)applicationDidFinishLaunching:(NSNotification *)note
{
    int i;

    [self setupMainMenu];
    [self windowSetUp];
    [UroPiece setBoardWidth:BWidth height:BHeight];

    for (i = 0; i < NumberOfClients; i++)
        [self clientSetUp];
    [uWindow makeKeyAndOrderFront:self];
}

@end
